#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 2.buildworld.sh,v 1.38 2005/06/17 16:49:03 saturnero Exp $
# vim: set ts=4:
#
# (obligatory) cd $SRCDIR && make buildworld ;) This step is 
# optional only if a buildworld (without clean) has been already 
# done, and above all if sources have not been updated after this 
# buildworld. If you didn't understand a word, do it! :)

set -e -u

. ./config.sh
. ./.common.sh

if [ ! -z "${NO_BUILDWORLD:-}" ]; then
    sh 3.installworld.sh
    return 0
fi

if [ "x$MINIMAL" = "xYES" ]; then
	export __MAKE_CONF=$LOCALDIR/files/make.conf.minimal # Redefine make.conf path
else
  if [ -z "${BATCH_MODE:-}" ]; then
	dialog --clear --title "FreeSBIE Live CD - Buildworld" --msgbox "We're now going to build \"world\". You can prevent the  build of some system components by customizing $LOCALDIR/files/make.conf . Please refer to the make.conf(5) man page for info about the format of this file." 12 35
  fi
	export __MAKE_CONF=$LOCALDIR/files/make.conf # Redefine make.conf path
fi

#Create a temporary log file
TEMP_FILE=`mktemp -q /tmp/freesbieXXXXXX`

#Trap ctrl+c and delete TEMP_FILE
trap "rm -f $TEMP_FILE && exit 1" INT

echo "Building world"
echo "This may take a while, go and watch the Star Wars Saga"
echo "The complete output of this script will be logged in $TEMP_FILE"

cd ${SRCDIR:-/usr/src} || print_error
if ! echo ${MAKEOPT} | egrep -q 'NOCLEAN|NO_CLEAN'; then
	echo "Cleaning build files..."
	rm -rf ${MAKEOBJDIRPREFIX:-/usr/obj}${SRCDIR:-/usr/src} || print_error
	(script -aq $TEMP_FILE make cleandir || print_error) | grep '^>>>'
fi
echo "Buildworld..."
(script -aq $TEMP_FILE $MAKE $MAKEJ buildworld $MAKEOPT || print_error) | grep '^>>>'
rm -f $TEMP_FILE

echo "[DONE]"

cd $LOCALDIR
[ ! -z "${BATCH_MODE:-}" ] && sh 3.installworld.sh
return 0
