#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 4.kernel.sh,v 1.43 2005/06/30 14:43:21 saturnero Exp $
# vim: set ts=4:
#
# (obligatory) Compiles and installs the personalized kernel 

set -e -u

. ./config.sh
. ./.common.sh

export KERNCONFDIR=${KERNCONFDIR:-"$LOCALDIR/files"}

if [ "x$MINIMAL" = "xYES" ]; then
	export __MAKE_CONF=$LOCALDIR/files/make.conf.minimal # Redefine make.conf path
else
	export __MAKE_CONF=$LOCALDIR/files/make.conf # Redefine make.conf path
fi

cp ${SRCDIR:-/usr/src}/sys/$ARCH/conf/GENERIC.hints $FREESBIEBASEDIR/boot/device.hints || print_error

if [ "x$MINIMAL" = "xYES" -a -z "${KERNCONF:-}" -a -z "${BATCH_MODE:-}" ]; then
	KERNCONF=FREESBIE.$ARCH
	echo -n "Do you want to edit the kernel file? [N/y]: "
	read ANSWER
	if [ "${ANSWER}" = "Y" -o "${ANSWER}" = "y" ]; then
		if [ -z "${EDITOR}" ]; then
		EDITOR=/usr/bin/ee
	fi
	[ -f $KERNCONFDIR/$KERNCONF.mini ] || cp $KERNCONFDIR/$KERNCONF $KERNCONFDIR/$KERNCONF.mini
	KERNCONF=$KERNCONF.mini
	${EDITOR} $KERNCONFDIR/$KERNCONF 
	fi
fi

ARCH=${ARCH:-`uname -m`}
export KERNCONF=${KERNCONF:-"FREESBIE.$ARCH"}

#Create a temporary log file
TEMP_FILE=`mktemp -q /tmp/freesbieXXXXXX`

#Trap ctrl+c and delete TEMP_FILE
trap "rm -f $TEMP_FILE && exit 1" INT

echo "To see what this script is doing, type:"
echo "tail -f $TEMP_FILE"

cd ${SRCDIR:-/usr/src} || print_error

if [ ! -z "${NO_BUILDKERNEL:-}" ]; then
	if [ ! -f /usr/obj/kerneldone.$KERNCONF ]; then
		echo ">>> Building FreeSBIE kernel..."
		(script -aq $TEMP_FILE $MAKE $MAKEJ buildkernel DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>'
		touch /usr/obj/kerneldone.$KERNCONF
		(script -aq $TEMP_FILE $MAKE $MAKEJ installkernel DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>'
	else 
		echo ">>> Kernel already built.  Installing (rm /usr/obj/kerneldone to bypass) ..."
		(script -aq $TEMP_FILE $MAKE $MAKEJ installkernel DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>'
	fi
else
	(script -aq $TEMP_FILE $MAKE $MAKEJ kernel DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>' 
fi

echo hint.psm.0.flags=0x1000 >> $FREESBIEBASEDIR/boot/device.hints || print_error

if [ "x$MINIMAL" = "xYES" -a "$ARCH" != "powerpc" ]; then
	echo "Compiling acpi"
	cd ${SRCDIR:-/usr/src}/sys/modules/acpi
	(script -aq $TEMP_FILE $MAKE depend DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>'  && \
	(script -aq $TEMP_FILE $MAKE DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>' && \
	(script -aq $TEMP_FILE $MAKE install DESTDIR=$FREESBIEBASEDIR $MAKEOPT || print_error) | grep '^>>>'
fi

rm -f $TEMP_FILE

cd $FREESBIEBASEDIR/boot/kernel || print_error
if [ "$ARCH" = "i386" ]; then strip kernel || print_error; fi
gzip -f9 kernel || print_error

echo "[DONE]"

cd $LOCALDIR
[ ! -z "${BATCH_MODE:-}" ] && sh 5.patchfiles.sh
return 0
