#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 5.patchfiles.sh,v 1.49 2005/07/06 22:09:18 drizzt Exp $
# vim: set ts=4:
#
# (obligatory) Copies configurations files required for a correct boot.

set -e -u

. ./config.sh
. ./.common.sh

[ -d $LOCALDIR/work ] && rm -rf $LOCALDIR/work

#Trap ctrl+c and delete work dir
trap "rm -rf $LOCALDIR/work && exit 1" INT

mkdir $LOCALDIR/work

touch $LOCALDIR/work/tmp_scriptchoice
if [ "$ARCH" != "powerpc" ]; then

if [ -z "${BATCH_MODE:-}" ]; then
    dialog --title "FreeSBIE Boot Scripts Selection" \
	--hline "Press F1 for Scripts Description" \
	--hfile files/dialogs/scripts_help \
	--checklist "Choose the scripts you want to include in FreeSBIE.\nPlease note that if you don't select mount_disks, the load/save settings\nfeature will be disabled" 12 77 5 \
	mount_disks "Mount slices in read-only mode at bootime"	on \
	save_settings "Save user settings" on \
	load_settings "Load user settings from a slice." on \
	sound_detect "Detect sound card and load the appropriate module" on \
	card_detect "Detect video card and configure XF86Config" on \
	2> $LOCALDIR/work/tmp_scriptchoice
else
    echo '"mount_disks" "save_settings" "load_settings" "sound_detect" "card_detect"' > $LOCALDIR/work/tmp_scriptchoice
fi

fi

mkdir -p $FREESBIEBASEDIR/scripts || print_error

echo "Copying FreeSBIE boot scripts"

cd $LOCALDIR/files/scripts

#install selected scripts
for i in `awk '{for (i=1; i<=NF;i++) {print substr($i,2,length($i)-2) }}' $LOCALDIR/work/tmp_scriptchoice`
	do
		if [ $i = "mount_disks" -o $i = "save_settings" -o $i = "load_settings" ]; then
			cp -p $LOCALDIR/files/scripts/$i.sh \
			$FREESBIEBASEDIR/scripts/$i.sh || print_error
	fi
	if [ $i = "sound_detect" ]; then
			mkdir -p $FREESBIEBASEDIR/scripts/sound || print_error
			for j in `find sound -type f -maxdepth 1`
				do
				cp -p $j $FREESBIEBASEDIR/scripts/$j || print_error
			done
	fi
	
	if [ $i = "card_detect" ]; then
			mkdir -p $FREESBIEBASEDIR/scripts/cardDetect || print_error
			for j in `find cardDetect -type f -maxdepth 1`
				do
				cp -p $j $FREESBIEBASEDIR/scripts/$j || print_error
			done
	fi
done

rm -r $LOCALDIR/work

#copy other scripts and files
mkdir -p $FREESBIEBASEDIR/scripts/personalData
for j in `find personalData -type f -maxdepth 1`
	do
	cp -p $j $FREESBIEBASEDIR/scripts/$j || print_error
done

#install man page
cp $LOCALDIR/files/freesbie.1 $FREESBIEBASEDIR/usr/share/man/man1/ || print_error

#patch $FREESBIEBASEDIR/etc
echo "Patching files in /etc directory..."
cp $LOCALDIR/files/rc.conf $FREESBIEBASEDIR/etc/rc.conf || print_error
mkdir -p $FREESBIEBASEDIR/usr/local/etc/rc.d || print_error
touch $FREESBIEBASEDIR/etc/fstab || print_error
if [ -z "${NO_MD:-}" ]; then
  cp $LOCALDIR/files/freesbie_1st $FREESBIEBASEDIR/etc/rc.d/freesbie_1st || print_error
  cp $LOCALDIR/files/rc.local $FREESBIEBASEDIR/etc/rc.local || print_error
  cp $LOCALDIR/files/motd $FREESBIEBASEDIR/etc || print_error
  cp $LOCALDIR/files/hosts $FREESBIEBASEDIR/etc || print_error
  cp $LOCALDIR/files/loader.conf $FREESBIEBASEDIR/boot || print_error
fi

#customize $FREESBIEBASEDIR/etc/rc.local according to installed scripts

if [ -f $FREESBIEBASEDIR/scripts/cardDetect/1.cardDetect.sh ]; then
	echo "#Video Card Detection" >> $FREESBIEBASEDIR/etc/rc.local
	echo "echo \"Detecting X...\"" >> $FREESBIEBASEDIR/etc/rc.local
	echo "if [ -f /usr/X11R6/bin/X ]; then" >> $FREESBIEBASEDIR/etc/rc.local 
	echo "	 echo \"X detected, creating XF86Config...\"" >> $FREESBIEBASEDIR/etc/rc.local
	echo "	 /scripts/cardDetect/1.cardDetect.sh" >> $FREESBIEBASEDIR/etc/rc.local
	echo "fi" >> $FREESBIEBASEDIR/etc/rc.local 
	echo "" >> $FREESBIEBASEDIR/etc/rc.local
fi

if [ -f $FREESBIEBASEDIR/scripts/sound/sound_detect.sh ]; then
	echo "#Sound module autodetection and load" >> $FREESBIEBASEDIR/etc/rc.local
	echo "/FreeSBIE/scripts/sound/sound_detect.sh" >> $FREESBIEBASEDIR/etc/rc.local
	echo "" >> $FREESBIEBASEDIR/etc/rc.local
fi

if [ -f $FREESBIEBASEDIR/scripts/mount_disks.sh ]; then
	echo "#Autoload in read only mode of all detected partitions" >> $FREESBIEBASEDIR/etc/rc.local
	echo "/FreeSBIE/scripts/mount_disks.sh ro" >> $FREESBIEBASEDIR/etc/rc.local
	echo "" >> $FREESBIEBASEDIR/etc/rc.local
fi

if [ -f $FREESBIEBASEDIR/scripts/load_settings.sh -a -f $FREESBIEBASEDIR/scripts/mount_disks.sh ]; then
	echo "#Try to load previously saved settings" >> $FREESBIEBASEDIR/etc/rc.local
	echo "/FreeSBIE/scripts/load_settings.sh" >> $FREESBIEBASEDIR/etc/rc.local
	echo "" >> $FREESBIEBASEDIR/etc/rc.local
fi

if [ -z "${BATCH_MODE:-}" ]; then
    if dialog --title "FreeSBIE rc.conf configuration" --yesno "Would you like to configure FreeSBIE /etc/rc.conf? (Please note that some options are mandatory and will be inserted anyway)" 8 60; then
	if [ ! -x $LOCALDIR/files/fishSBIE/fishSBIE ]; then
		echo "[Building fishSBIE]"
		cd $LOCALDIR/files/fishSBIE || print_error
		make || print_error
		make clean || print_error
	fi
	$LOCALDIR/files/fishSBIE/fishSBIE -f $FREESBIEBASEDIR/etc/rc.conf -d \
	$FREESBIEBASEDIR/etc/defaults/rc.conf || print_error
    fi

    if dialog --title "FreeSBIE loader.conf configuration" --yesno "Would you like to configure FreeSBIE /boot/loader.conf ? (Please note that some options are mandatory and will be inserted anyway)" 8 60 ; then
	if [ ! -x $LOCALDIR/files/fishSBIE/fishSBIE ]; then
		cd $LOCALDIR/files/fishSBIE || print_error
		make || print_error
		make clean || print_error
	fi
	$LOCALDIR/files/fishSBIE/fishSBIE -f $FREESBIEBASEDIR/boot/loader.conf -d \
	$FREESBIEBASEDIR/boot/defaults/loader.conf || print_error

	grep -v "#optional$" $LOCALDIR/files/loader.conf | \
	while read i
	do
		VAR=`echo $i | cut -f 1 -d =`
		
		if grep "^$VAR=" $FREESBIEBASEDIR/boot/loader.conf | grep -vq "^$i\$"; then
			SECOND=`grep "^$VAR=" $FREESBIEBASEDIR/boot/loader.conf | grep -v "^$i\$" | cut -f 2 -d =`
			sed -i "" "/^$VAR=$SECOND\$/d" $FREESBIEBASEDIR/boot/loader.conf
		fi
	done
    fi
fi

cd $LOCALDIR
set +u
[ ! -z "${BATCH_MODE:-}" ] && sh 6.packages.sh
return 0
