#!/bin/sh
# $Id: 6.packages.sh,v 1.27 2005/06/22 23:06:26 saturnero Exp $
# vim: set ts=4:
#
# (optional) The user is allowed to choose from a list of installed packages, 
# packages to add in the LiveCD. Dependecies are automatically found and
# installed. 
# Watch out for some particular cases[...]

. ./config.sh

# Trap the ctrl+c and delete the temporary files
trap "cd .. && rm -rf work && exit 1" INT

DIR_TEMP=temp
FILE_CAT=packages.sh

cd $LOCALDIR

rm_work() {
  [ -d work ] && rm -rf work
  mkdir -p work/${DIR_TEMP}
}

create_lists() {
  cd $LOCALDIR/work
  echo -n "Creating list of installed packages in your REAL system... "
  pkg_info -Qoa | awk -v tempdir=${DIR_TEMP} \
	'{
		split($1,info,":");
		split(info[2],tcat,"/");
		system("echo \"\\\"" info[1] "\\\" \\\"\\\" off \\\\\" >> " tempdir "/" tcat[1] ".src");
	 }'
	 
  echo "[OK]"
  CATEGORIES=`basename -s ".src" ${DIR_TEMP}/*.src`

  echo -n "Creating list of installed packages in your LIVE system... "
  [ -f tlist ] && rm -f tlist
  chroot ${FREESBIEBASEDIR} pkg_info -Qoa | awk -v tempdir=${DIR_TEMP} \
	'{
		split($1,info,":");
		split(info[2],tcat,"/");
		system("echo " info[1] " >> tlist");
			 system("echo \"\\\"" info[1] "\\\" \\\"\\\" on \\\\\" >> " tempdir "/" tcat[1] ".lst");
		 
	 }'
  echo "[OK]"
  cd $LOCALDIR
}

do_dialog() {
  cd work
  FLAG=true
  while [ "${FLAG}" = "true" ]
  do
	echo "/usr/bin/dialog --title \"FreeSBIE LiveCD - Packages\" --clear \
	--menu \"These are the packages installed on your system \n\
	Choose the FreeSBIE packages\" -1 -1 10 \\" > $FILE_CAT
	echo "Save and exit" | awk '{print "\"" $0 "\" \"\" \\"}' >> $FILE_CAT
	echo " " | awk '{print "\"" $0 "\" \"\" \\"}' >> $FILE_CAT
	for i in $CATEGORIES
	do
		echo $i | awk '{print "\"" $1 "\" \"\" \\"}' >> $FILE_CAT
	done
	echo "2> /tmp/checklist.tmp.$$" >> $FILE_CAT
	echo "retval=\$?" >> $FILE_CAT
	echo "case \$retval in" >> $FILE_CAT 
	echo "  0)" >> $FILE_CAT
	echo "echo \"0\" > /tmp/result.tmp.$$ ;;" >> $FILE_CAT
	echo "  1)" >> $FILE_CAT
	echo "echo \"1\" > /tmp/result.tmp.$$ ;;" >> $FILE_CAT
	echo "  255)" >> $FILE_CAT
	echo "echo \"255\" > /tmp/result.tmp.$$ ;;" >> $FILE_CAT
	echo "esac" >> $FILE_CAT
	sh $FILE_CAT
	choice=`cat /tmp/checklist.tmp.$$`
	rm -f /tmp/checklist.tmp.$$
	if [ "$choice" = " " ]; then
		continue
	elif [ "$choice" = "Save and exit" ]; then
		echo -n "Saving preferences..."
		if [ -e plist ]; then
			rm plist
		fi
		for i in ${DIR_TEMP}/*.lst
		do
			echo $i	
			awk '{print substr($1,2,length($1)-2)}' $i >> plist.tmp
		done
	
		sort -o plist plist.tmp 
		rm plist.tmp
		if [ -r ${FILE_CAT} ]; then
			rm ${FILE_CAT}
		fi
		echo " [OK]"
		#
		#	Creating list of packages not installed and list of packages to
		#	delete 
		#
		[ -r to_delete ] &&  rm to_delete
		[ -r to_install ] && rm to_install
		while read i	
		do
			if [ -f tlist ]; then
				RESULT_A=`grep $i tlist`
			else
				RESULT_A=""
			fi
			if [ -z "${RESULT_A}" ]; then
				echo $i >> to_install
			fi
		done < plist
		if [ -f tlist ]; then
			while read i	
			do
				RESULT_B=`grep $i plist`
				if [ -z "${RESULT_B}" ]; then
					echo $i >> to_delete
				fi
			done < tlist
		fi
		FLAG="false"
		break	
	fi
	loop=`cat /tmp/result.tmp.$$`
	rm -f /tmp/result.tmp.$$
	if [ ! "${loop}" = "0" ]; then
		exit
	fi
	echo "#!/bin/sh" > ${DIR_TEMP}/${choice}.sh
	echo "/usr/bin/dialog --title \"FreeSBIE LiveCD - Packages\" --clear \
	--checklist \"These are the packages installed on your system \n\
	Choose the FreeSBIE packages\" -1 -1 10 \\" >> ${DIR_TEMP}/${choice}.sh
	
	if [ -e ${DIR_TEMP}/${choice}.lst ]; then
		PRESENT=`awk '{print $1}' ${DIR_TEMP}/${choice}.src`
		FUTURE=`awk '{for (i=1;i<=NF;i++) print $i;}' ${DIR_TEMP}/${choice}.lst`
		echo $FUTURE > ${DIR_TEMP}/packages_future.tmp
		if [ -e ${DIR_TEMP}/${choice}.dst ]; then
			rm ${DIR_TEMP}/${choice}.dst
		fi
		for j in ${PRESENT}
		do
			RESULT=`grep ${j} ${DIR_TEMP}/packages_future.tmp`
			if [ -z "${RESULT}" ]; then
				echo "${j} \"\" off \\" >> ${DIR_TEMP}/${choice}.sh
			else
				echo "${j} \"\" on \\" >> ${DIR_TEMP}/${choice}.sh
			fi
		done
		rm  ${DIR_TEMP}/packages_future.tmp
	else
		cat ${DIR_TEMP}/${choice}.src >> ${DIR_TEMP}/${choice}.sh
		cp ${DIR_TEMP}/${choice}.src ${DIR_TEMP}/${choice}.lst
	fi
	echo "2> ${DIR_TEMP}/${choice}.lst.tmp1" >> ${DIR_TEMP}/${choice}.sh
	echo "if [ \$? = 0 ]; then " >> ${DIR_TEMP}/${choice}.sh
	echo "   cat ${DIR_TEMP}/${choice}.lst.tmp1 > ${DIR_TEMP}/${choice}.lst.tmp" >> ${DIR_TEMP}/${choice}.sh
	echo "else" >> ${DIR_TEMP}/${choice}.sh
	echo "   touch  ${DIR_TEMP}/${choice}.lst.tmp" >> ${DIR_TEMP}/${choice}.sh
	echo "fi"  >> ${DIR_TEMP}/${choice}.sh
	echo "rm ${DIR_TEMP}/${choice}.lst.tmp1" >> ${DIR_TEMP}/${choice}.sh

	sh ${DIR_TEMP}/${choice}.sh
	if [ -f ${DIR_TEMP}/${choice}.lst.tmp ]; then
		awk '{for (i=1; i<=NF;i++) {print $i " \"\" on \\"}}' ${DIR_TEMP}/${choice}.lst.tmp > ${DIR_TEMP}/${choice}.lst
		rm ${DIR_TEMP}/${choice}.lst.tmp
	fi
  done
}

delete_packages() {
  if [ -f work/to_delete ]; then
	echo "Deleting packages... "
	DELLIST=""
	while read i; do DELLIST="$DELLIST $i"; done < work/to_delete
	for i in `pkg_info -qr $DELLIST | cut -d ' ' -f2 | sort | uniq`; do
		RESULT_A=`echo $DELLIST | grep $i`
	[ -z "$RESULT_A" ] && DELLIST="$DELLIST $i"; 
	done
	chroot $FREESBIEBASEDIR pkg_delete $DELLIST;
	echo "[OK]"
fi
}

generate_pkg_info() {
	chroot $FREESBIEBASEDIR pkg_info > $FREESBIEBASEDIR/pkg_info.txt
	chroot $FREESBIEBASEDIR pkg_info -Ea | awk '{ gsub("-[^-]+$", ""); print;}' > $FREESBIEBASEDIR/var/db/pkg/pkg_names.txt
	chroot $FREESBIEBASEDIR pkg_info -oaq | sort > $FREESBIEBASEDIR/var/db/pkg/pkg_origins.txt
}

install_packages() {
  if [ -f ${PORTS_FILE} ]; then
	echo "Creating packages to install: "
	ADDLIST=""
	while read i; do 
	  p=`echo $i | cut -d\  -f 1`; 
	  j=`pkg_info -EX "${p}($|-[^-]+$)"`;
	  ADDLIST="$ADDLIST $j"; 
	done < ${PORTS_FILE}
	for i in `pkg_info -qr $ADDLIST | cut -d ' ' -f2 | sort | uniq`; do
		RESULT_A=`echo $ADDLIST | grep $i`
	if [ -f tlist ]; then
		RESULT_B=`grep $i tlist`
	else
		RESULT_B=""
	fi
	[ -z "$RESULT_A" -a -z "$RESULT_B" ] && ADDLIST="$ADDLIST $i"
	done		
	for i in `pkg_info -qr $ADDLIST | cut -d ' ' -f2 | sort | uniq`; do
		RESULT_A=`echo $ADDLIST | grep $i`
	if [ -f tlist ]; then
		RESULT_B=`grep $i tlist`
	else
		RESULT_B=""
	fi
	[ -z "$RESULT_A" -a -z "$RESULT_B" ] && ADDLIST="$ADDLIST $i" 
	done
	BACK=$PWD; cd $FREESBIEBASEDIR
  	rm -f $FREESBIEBASEDIR/*.tgz
	for i in $ADDLIST; do echo -n "  Creating $i... "; [ ! -f $i.tgz ] && pkg_create -zb $i; echo "[OK]"; done
	cd $BACK
	echo -n "Installing packages... "
	while read i; do j=`echo $i | cut -d\  -f 1`; chroot $FREESBIEBASEDIR pkg_add $j.tgz; done < ${PORTS_FILE}
        rm -f $FREESBIEBASEDIR/*.tgz
	echo "[OK]"
  fi
}

clean_out() {
  echo -n "Cleaning out working directory... "
  rm -rf $LOCALDIR/work
  echo "[OK]"
}

if [ $# -eq 1 ]; then
  PORTS_FILE=$1
else
  if [ ! -z "$PKG_FILE" ]; then
    PORTS_FILE=$PKG_FILE
  fi
fi

if [ ! -z "$PORTS_FILE" -o -z "${BATCH_MODE:-}" ]; then
    rm_work
    create_lists
    if [ -z "$PORTS_FILE" ]; then
	do_dialog
	PORTS_FILE="${LOCALDIR}/work/to_install"
	delete_packages
    fi
    install_packages
    generate_pkg_info
    clean_out
fi

cd $LOCALDIR
[ ! -z "${BATCH_MODE:-}" ] && sh 7.customuser.sh
return 0
