#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 7.customuser.sh,v 1.49 2005/06/18 02:51:32 saturnero Exp $
# vim: set ts=4:
#
# (obligatory) This script should be useful to install known workarounds 
# to make some packages work.

set -u

. ./config.sh
. ./.common.sh

cp $LOCALDIR/COPYING $FREESBIEBASEDIR/COPYRIGHT.FreeSBIE || print_error

echo "Creating default user and installing customized files..."

#Modifing root shell
chroot $FREESBIEBASEDIR pw usermod root -s /bin/tcsh || print_error

#Preventing pw to create the symlink home -> /usr/home, we well do it later
if [ -L $FREESBIEBASEDIR/home -a -d $FREESBIEBASEDIR/usr/home ]; then
    rm $FREESBIEBASEDIR/home
    mv $FREESBIEBASEDIR/usr/home $FREESBIEBASEDIR/
else
    mkdir -p $FREESBIEBASEDIR/home
fi

#Adding default group
grep -q '^freesbie' $FREESBIEBASEDIR/etc/group
if [ $? -ne 0 ]; then
    chroot $FREESBIEBASEDIR pw group add freesbie -g 1000
fi

#Adding default user with no password
grep -q freesbie $FREESBIEBASEDIR/etc/master.passwd

if [ $? -ne 0 ]; then
    chroot $FREESBIEBASEDIR pw useradd freesbie \
	-u 1000 -c "FreeSBIE User" -d "/home/freesbie" \
	-g 1000 -G 0,5 -m -s /bin/tcsh -k /usr/share/skel -w nonex
else
    chroot $FREESBIEBASEDIR pw usermod freesbie \
	-u 1000 -c "FreeSBIE User" -d "/home/freesbie" \
	-g 1000 -G 0,5 -m -s /bin/tcsh -k /usr/share/skel -w none
fi

#Copying user diff for the root filesystem
cd $LOCALDIR/files/custom
find . -not -name 'README' -not -path '*CVS*' | cpio --quiet -pudmR 0:0 $FREESBIEBASEDIR 2>&1 > /dev/null

#Move home directory under /usr and fix freesbie user permission
if [ ! -L $FREESBIEBASEDIR/home -a -d $FREESBIEBASEDIR/home ]; then
    mv $FREESBIEBASEDIR/home $FREESBIEBASEDIR/usr/
    ln -s usr/home $FREESBIEBASEDIR/
    grep -q ^freesbie: $FREESBIEBASEDIR/etc/master.passwd && \
      chroot $FREESBIEBASEDIR /usr/sbin/chown -R freesbie:freesbie /home/freesbie
fi

if [ -f $FREESBIEBASEDIR/usr/X11R6/bin/XFree86 ]; then
	echo "X installed, creating /etc/X11/..."
	mkdir -p $FREESBIEBASEDIR/etc/X11/ || print_error
	rm $FREESBIEBASEDIR/usr/X11R6/bin/X  || print_error
	ln -sf $FREESBIEBASEDIR/usr/X11R6/bin/XFree86 $FREESBIEBASEDIR/usr/X11R6/bin/X || print_error
fi

if [ "x$MINIMAL" = "xYES" ]; then
	rm -rf \
	$FREESBIEBASEDIR/usr/share/nls \
	$FREESBIEBASEDIR/usr/share/bsnmp \
	$FREESBIEBASEDIR/usr/share/dict \
	$FREESBIEBASEDIR/usr/share/doc \
	$FREESBIEBASEDIR/usr/share/examples \
	$FREESBIEBASEDIR/usr/share/games \
	$FREESBIEBASEDIR/usr/share/isdn \
	$FREESBIEBASEDIR/usr/share/locale \
	$FREESBIEBASEDIR/usr/share/me \
	$FREESBIEBASEDIR/usr/share/pcvt \
	$FREESBIEBASEDIR/usr/share/security \
	$FREESBIEBASEDIR/usr/share/sendmail \
	$FREESBIEBASEDIR/usr/share/bsnmp \
	$FREESBIEBASEDIR/usr/share/calendar/*/ \
	$FREESBIEBASEDIR/usr/include/*

fi

cd $LOCALDIR
set +u
[ ! -z "${BATCH_MODE:-}" ] && sh 71.bsdinstaller.sh
return 0
