#!/bin/sh
#
# Copyright 2002-2005 FreeSBIE Project and BSD Installer Project.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 71.bsdinstaller.sh,v 1.1 2005/06/18 01:57:09 saturnero Exp $
# vim: set ts=4:
#
# (obligatory) This script should be useful to install known workarounds 
# to make some packages work.

set -u

. ./config.sh
. ./.common.sh

if [ "$BSDINSTALLER" = "YES" ]; then
	mkdir -p $FREESBIEBASEDIR/usr/local/share/dfuibe_installer
	cp $LOCALDIR/files/sources.conf $FREESBIEBASEDIR/usr/local/share/dfuibe_installer/sources.conf
	SITEURL="http://www.bsdinstaller.com/packages/FBSD5"
	PACKAGES="libaura~libaura-latest.tbz \
	libdfui~libdfui-latest.tbz \
	libinstaller~libinstaller-latest.tbz \
	dfuibe_installer~dfuibe_installer-latest.tbz \
	dfuife_cgi~dfuife_cgi-latest.tbz \
	thttpd~thttpd-notimeout-latest.tbz \
	cpdup~cpdup-latest.tbz \
	dfuife_curses~dfuife_curses-latest.tbz"

	if [ -e $FREESBIEBASEDIR/chrootscript ]; then
		rm $FREESBIEBASEDIR/chrootscript
	fi
	for PKG in ${PACKAGES}; do
		pkg_name=`echo $PKG | cut -d~ -f 1`
		pkg_url=`echo $PKG | cut -d~ -f 2`
		echo "Checking for $pkg_name ..."
		does_exist=`chroot $FREESBIEBASEDIR pkg_info | grep $pkg_name`
		if [ -n "$does_exist" ]; then
			echo "Package $pkg_name already exists.  Skipping."
		else
			# Fetch package from BSDInstaller.com and install
			echo "Downloading package $SITEURL/$pkg_url ..."
			cd $FREESBIEBASEDIR && fetch "$SITEURL/$pkg_url"
			echo pkg_add /$pkg_url >> $FREESBIEBASEDIR/chrootscript
			echo rm /$pkg_url >> $FREESBIEBASEDIR/chrootscript
			echo ldconfig -m /usr/local/lib >> $FREESBIEBASEDIR/chrootscript
		fi
	done
	echo "Chrooting into $FREESBIEBASEDIR and running /chrootscript ..."
	if [ -f "$FREESBIEBASEDIR/chrootscript" ]; then
		chroot $FREESBIEBASEDIR sh /chrootscript
		rm $FREESBIEBASEDIR/chrootscript
	fi
	cp $LOCALDIR/files/scripts/after_installation_routines.sh $FREESBIEBASEDIR/usr/local/bin/
	chmod a+x $FREESBIEBASEDIR/usr/local/bin/after_installation_routines.sh
	cp $LOCALDIR/files/scripts/installer $FREESBIEBASEDIR/scripts/
	chmod a+x $FREESBIEBASEDIR/scripts/installer
	cp $LOCALDIR/files/scripts/web_installer $FREESBIEBASEDIR/scripts/
	chmod a+x $FREESBIEBASEDIR/scripts/web_installer
fi

cd $LOCALDIR
set +u
[ ! -z "${BATCH_MODE:-}" ] && sh 8.preparefs.sh
return 0
