#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 8.preparefs.sh,v 1.44 2005/07/06 22:09:18 drizzt Exp $	
#
# (obligatory) This script creates the MFS root image for booting the
# ISO and creates compressed loop filesystem of /usr and /var
# directories. They will be uncompressed and mounted at boot-time

set -u

. ./config.sh
. ./.common.sh

cd $LOCALDIR

#Create a copy of FREESBIEBASEDIR. This is useful to modify the live filesystem
clone_system()
{
  echo -n "Cloning $FREESBIEBASEDIR to $FREESBIEISODIR..."

  mkdir -p $FREESBIEISODIR || print_error
  if [ -r $FREESBIEISODIR ]; then
        chflags -R noschg $FREESBIEISODIR || print_error
        rm -rf $FREESBIEISODIR/* || print_error
  fi
 
  if [ -z "${NO_MD:-}" ]; then
  
    #We are making files containing /usr and /var partition
                                
    #Before uzip'ing filesystems, we have to save the directories tree
    mkdir -p $FREESBIEISODIR/dist
    mtree -Pcdp $FREESBIEBASEDIR/usr > $FREESBIEISODIR/dist/FreeSBIE.usr.dirs
    mtree -Pcdp $FREESBIEBASEDIR/var > $FREESBIEISODIR/dist/FreeSBIE.var.dirs

    #Define a function to create the vnode $1 of the size expected for
    #$FREESBIEBASEDIR/$2 directory, mount it under $FREESBIEISODIR/$2
    #and print the md device
    create_vnode() {
      UFSFILE=$1
      CLONEDIR=$FREESBIEBASEDIR/$2
      MOUNTPOINT=$FREESBIEISODIR/$2
      cd $CLONEDIR
      FSSIZE=$((`du -kd 0 | cut -f 1` + 64000))
      dd if=/dev/zero of=$UFSFILE bs=1k count=$FSSIZE > /dev/null 2>&1 

      DEVICE=/dev/`mdconfig -a -t vnode -f $UFSFILE`
      newfs $DEVICE > /dev/null 2>&1 
      mkdir -p $MOUNTPOINT
      mount -o noatime ${DEVICE} $MOUNTPOINT
      echo ${DEVICE}
    }

    #Umount and detach md devices passed as parameters
    umount_devices() {
      echo -n "Umounting loop devices..."
      for i in $@; do
	  umount ${i}
	  mdconfig -d -u ${i}
      done
      echo " done"
    }

    mkdir -p $FREESBIEISODIR/uzip
    MDDEVICES=`create_vnode $FREESBIEISODIR/uzip/usr.ufs usr`  
    MDDEVICES="$MDDEVICES `create_vnode $FREESBIEISODIR/uzip/var.ufs var`"
  
    trap "umount_devices $MDDEVICES; exit 1" INT

  fi
  cd $FREESBIEBASEDIR
  if [ -z "${FILE_LIST:-}" ]; then
      find . -print -depth | cpio --quiet -pudm $FREESBIEISODIR
  else
      # XXX - Should insert a check if $FILE_LIST exists or not
      cat $FILE_LIST | cpio --quiet -pudm $FREESBIEISODIR
  fi

  if [ -z "${NO_MD:-}" ]; then
    umount_devices $MDDEVICES

    trap "" INT
  fi
  
  echo " [DONE]"
}

#Create memory filesystem root image
create_mfsroot()
{
  echo -n "Creating MFSROOT... "
  roundup() echo $((($1+$2-1)-($1+$2-1)%$2))

  MNT=`mktemp -d /tmp/mnt.X`
  COPYDIR="$FREESBIEISODIR/rescue $FREESBIEISODIR/etc $FREESBIEISODIR/root $FREESBIEISODIR/proc"
  NDDIRS="uzip|dist|scripts|boot|usr|var|bin|sbin|lib|libexec"

  FSIMAGE=$FREESBIEISODIR/boot/mfsroot

  sk=$(du -sk $COPYDIR | cut -f1 | xargs | sed 's/ /+/g' | bc)
  FSSIZE=$(($(roundup $(($sk*12/10)) 1024)))

  dd of=$FSIMAGE if=/dev/zero count=$FSSIZE bs=1k >/dev/null 2>&1

  # load g_md kernel module
  if ! kldstat -v | grep -q g_md\$; then
		if kldload g_md; then
			echo "g_md module loaded."
		else
			echo "g_md module failed to load."
			exit 1
		fi
	fi

	MDDEVICE=`mdconfig -a -t vnode -f $FSIMAGE`
	
	if [ ! -c /dev/${MDDEVICE} ] ; then
		echo "No /dev/$MDDEVICE" 1>&2
		exit 1
	fi

	bsdlabel -w $MDDEVICE >/dev/null 2>&1
	newfs /dev/${MDDEVICE} >/dev/null 2>&1
	tunefs -m0 /dev/${MDDEVICE} >/dev/null 2>&1

	mount /dev/${MDDEVICE} $MNT

	mtree -deU -f $LOCALDIR/files/mfsroot.run.dist -p $MNT >/dev/null 2>&1

	ln -s /rescue $MNT/bin
	ln -s /rescue $MNT/sbin

	# We need to leave these files in place for the installer
	# if the user has selected this option since the installer
	# mounts up /usr and /var in /cdrom/ and proceeds with the
	# install from this location.
	for i in $COPYDIR
	do
		if [ ! -d $i ] ; then
			exit 1
		fi
		BASE=$MNT/`basename $i`
		mkdir $BASE
		tar -cpf - -C $i . | tar -xpf - -C $BASE
		if [ "$BSDINSTALLER" != "YES" ]; then
			chflags -R noschg $i
			rm -rf $i
		fi
	done
	
	umount $MNT
	mdconfig -d -u $MDDEVICE	
	rmdir	$MNT

	gzip -f9 $FSIMAGE

	# We need to leave these files in place for the installer
	# if the user has selected this option since the installer
	# mounts up /usr and /var in /cdrom/ and proceeds with the
	# install from this location.
	if [ "$BSDINSTALLER" != "YES" ]; then
		for i in `find $FREESBIEISODIR -type d -depth 1`
		do
			BASE=`basename $i`
			echo $BASE | egrep -qwv $NDDIRS
			if [ $? = 0 ]; then
				chflags -R noschg $i
				rm -rf $i
			fi
		done
	fi
    echo " [DONE]"

}


# Check that shared libraries are present for every dynamically linked binary 
check_dyn_linked() {
  echo "------------------------------------------------------------"
  echo "Now I'm going to check all dynamically linked ELF binaries,"
  echo "To prevent \"/libexec/ld-elf.so.1: Shared object "
  echo "\"libXXX.so.XX\" not found error."
  echo "------------------------------------------------------------"
  check_dyn_linked

  TO_CHECK_BIN_DIR="/usr/local/bin/ /usr/local/sbin/ /usr/X11R6/bin/ /usr/X11R6/sbin/" 
  BIN_DIR=""
  ERROR=""
  DIR=""
  CHCMD="chroot $FREESBIEISODIR"
  CHROOTLDD=`$CHCMD which ldd`
  
  for i in $TO_CHECK_BIN_DIR
  do
    [ -d $FREESBIEISODIR/$i ] && BIN_DIR="$BIN_DIR $i"
  done 
  
  for j in $BIN_DIR
  do
    echo "Parsing ${j}"
	for i in `$CHCMD find ${j} -type f`
		do
			FILE_TYPE=`$CHCMD file $i`
			IS_DYN=`echo ${FILE_TYPE} | grep dynamically`
			if [ "${IS_DYN}" ]; then 
				RETVAL=`$CHCMD $CHROOTLDD $i | grep "not found"`
				if [ "${RETVAL}" ]; then 
					LIB=`echo ${RETVAL} | cut -d\= -f 1`
					echo "$i has an unresolved dependency ( ${LIB}). "
					ERROR="Y"
				fi
			fi
		done
	done
	if [ "${ERROR}" = "Y" ]; then
      echo "FreeSBIE aborted because some programs you included have "
      echo "problems with their dependencies."
      echo "You have a lot of choices:"
      echo "- find the ports with dependencies problems and upgrade them;"
      echo "- copy your /usr/local/lib/compat/pkg to $FREESBIEBASEDIR;"
      echo "- watch a dvd and hope santa claus will solve your problems."
      exit 1
    fi
}

#Create an uziped filesystem named $2 starting from $1 file
uzip() {
    UFSFILE=$1
    UZIPFILE=$2

    echo -n "Compressing ${UFSFILE}..."
    mkuzip -s 65536 -o $UZIPFILE $UFSFILE
      
    [ "$?" != "0" ] && echo "Something went wrong while uzip'ing ${UFSFILE}." && exit 1

    UFSSIZE=`ls -l $UFSFILE | awk '{print $5}'`
    UZIPSIZE=`ls -l $UZIPFILE | awk '{print $5}'`

    PERCENT=`awk -v ufs=$UFSSIZE -v uzip=$UZIPSIZE 'BEGIN{print (100 - (100 * (uzip/ufs)));}'`
    rm -f $UFSFILE
      	
    echo " [DONE] ${PERCENT}% saved"

}

#Create compressed filesystem images
create_uzips() {
    #We are uziping /usr and /var
    uzip $FREESBIEISODIR/uzip/usr.ufs $FREESBIEISODIR/uzip/usr.uzip
    uzip $FREESBIEISODIR/uzip/var.ufs $FREESBIEISODIR/uzip/var.uzip
}

#Remove uneeded files
remove_files() {
  echo -n "Removing unwanted files... "
  rm -f $FREESBIEISODIR/etc/X11/XF86Config
  rm -f $FREESBIEISODIR/etc/resolv.conf
  rm -f $FREESBIEISODIR/root/.choose
  rm -f $FREESBIEISODIR/root/.first_time
  rm -f $FREESBIEISODIR/root/.xfirst_time
  rm -f $FREESBIEISODIR/root/.kbdmap_conf
  rm -f $FREESBIEISODIR/root/.message_done
  rm -f $FREESBIEISODIR/root/.xfree86
  rm -f $FREESBIEISODIR/root/.xorg_wm
  rm -f $FREESBIEISODIR/root/.ICEauthority
  echo "[DONE]"
}

clone_system
remove_files
if [ -z "${NO_MD:-}" ]; then
	create_mfsroot
	create_uzips
fi
	
cd $LOCALDIR
set +u
[ ! -z "${BATCH_MODE:-}" ] && sh 81.mkiso.sh
return 0
