#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: 9.restorefs.sh,v 1.7 2005/04/26 17:07:43 rionda Exp $
# vim: set ts=4:

. ./config.sh

if [ ! -x /usr/local/bin/extract_compressed_fs ]; then
	echo "Missing cloop utils. Please install sysutils/cloop-utils from port
s collection"
	exit 1
fi

if [ ! -f "$PATHISO" ]; then
	echo "Can't open $PATHISO: No such file or directory"
	exit 1
fi

MNT=`mktemp -d /tmp/mnt.X`
# load g_md kernel module
if ! kldstat -v | grep -q g_md\$; then
	if kldload g_md; then
		echo "g_md module loaded."
	else
		echo "g_md module failed to load."
		exit 1
	fi
fi

MDDEVICE="/dev/`mdconfig -a -t vnode -f $PATHISO`"

echo -n "Deleting old dir..."

chflags -R noschg $FREESBIEBASEDIR
rm -rf $FREESBIEBASEDIR
echo " done."

mkdir $FREESBIEBASEDIR

echo -n "Copying iso content..."
mount -t cd9660 $MDDEVICE $MNT

cd $MNT
find . -depth -print | cpio -pudm $FREESBIEBASEDIR

cd $FREESBIEBASEDIR

umount $MNT
mdconfig -d -u $MDDEVICE

echo " done."
echo "Uncompressing cloop files..."

USR_CLOOP=cloop/usr.cloop
USR_ISO=old_usr.iso

X11R6_CLOOP=cloop/x11r6.cloop
X11R6_ISO=old_x11r6.iso

OPENOFFICE_CLOOP=cloop/openoffice.cloop
OPENOFFICE_ISO=old_openoffice.iso

VAR_CLOOP=cloop/var.cloop
VAR_ISO=old_var.iso

MFS_GZIP=boot/mfsroot.gz
MFS=boot/mfsroot

cd $FREESBIEBASEDIR
/usr/local/bin/extract_compressed_fs ${USR_CLOOP} > ${USR_ISO}
/usr/local/bin/extract_compressed_fs ${VAR_CLOOP} > ${VAR_ISO}

[ -f ${X11R6_CLOOP} ] && \
    /usr/local/bin/extract_compressed_fs ${X11R6_CLOOP} > ${X11R6_ISO}

[ -f ${OPENOFFICE_CLOOP} ] && \
    /usr/local/bin/extract_compressed_fs ${OPENOFFICE_CLOOP} > ${OPENOFFICE_ISO}
rm ${USR_CLOOP} ${VAR_CLOOP}
[ -f ${X11R6_CLOOP} ] && rm ${X11R6_CLOOP}
[ -f ${OPENOFFICE_CLOOP} ] && rm ${OPENOFFICE_CLOOP}

gunzip -f $MFS_GZIP

echo -n "Restoring /usr cloop filesystem..."
MD_USR="/dev/`mdconfig -a -t vnode -f ${USR_ISO}`"
mount -t cd9660 ${MD_USR} $MNT
cd $MNT
find . -depth -print | cpio -pudm $FREESBIEBASEDIR/usr
cd $FREESBIEBASEDIR
umount $MNT
mdconfig -d -u ${MD_USR}
rm ${USR_ISO}
echo " done."

if [ -f ${X11R6_ISO} ]; then
    echo -n "Restoring /usr/X11R6 cloop filesystem..."
    MD_X11R6="/dev/`mdconfig -a -t vnode -f ${X11R6_ISO}`"
    mount -t cd9660 ${MD_X11R6} $MNT
    cd $MNT
    find . -depth -print | cpio -pudm $FREESBIEBASEDIR/usr/X11R6
    cd $FREESBIEBASEDIR
    umount $MNT
    mdconfig -d -u ${MD_X11R6}
    rm ${X11R6_ISO}
    echo " done."
fi

if [ -f ${OPENOFFICE_ISO} ]; then
    echo -n "Restoring /usr/local/OpenOffice.org1.1.3 cloop filesystem..."
    MD_OPENOFFICE="/dev/`mdconfig -a -t vnode -f ${OPENOFFICE_ISO}`"
    mount -t cd9660 ${MD_OPENOFFICE} $MNT
    cd $MNT
    find . -depth -print | cpio -pudm $FREESBIEBASEDIR/usr/local/OpenOffice.org1.1.3
    cd $FREESBIEBASEDIR
    umount $MNT
    mdconfig -d -u ${MD_OPENOFFICE}
    rm ${OPENOFFICE_ISO}
    echo " done."
fi

bzcat dist/FreeSBIE.usr.dist.bz2 | mtree -PUr -p usr
rm dist/FreeSBIE.usr.dist.bz2

echo "Restoring /var cloop filesystem..."
MD_VAR="/dev/`mdconfig -a -t vnode -f ${VAR_ISO}`"
mount -t cd9660 ${MD_VAR} $MNT
cd $MNT
find . -depth -print | cpio -pudm $FREESBIEBASEDIR/var
cd $FREESBIEBASEDIR
umount $MNT
mdconfig -d -u ${MD_VAR}
bzcat dist/FreeSBIE.var.dist.bz2 | mtree -PUr -p var
rm dist/FreeSBIE.var.dist.bz2
echo " done."

MD_MFS="/dev/`mdconfig -a -t vnode -f ${MFS}`"
mount ${MD_MFS} $MNT
cd $MNT
find . -depth -print | cpio -pudm $FREESBIEBASEDIR
cd $FREESBIEBASEDIR
umount $MNT
mdconfig -d -u ${MD_MFS}
bzcat dist/FreeSBIE.root.dist.bz2 | mtree -PUr -p .
rm dist/FreeSBIE.root.dist.bz2

rmdir $MNT
