#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
#$Id: load_settings.sh,v 1.6 2005/04/26 17:22:46 rionda Exp $

CONFIGFILE_NAME="freesbie_settings.conf"
CONFIG_DIR=""

set_defaultrouter() {
  route add default $DEFAULTROUTER >/dev/null 2>&1
}

set_hostname() {
  hostname $HOSTNAME
}

load_rc_conf() {
  #It's necessary copy back the rc.conf file because a lot of daemons
  #starts only if a specific directive is present in rc.conf, e.g.
  #ddclient_enable="YES" or apache_enable="YES". 
  cp $CONFIG_DIR/rc.conf /etc/rc.conf
}

load_ppp_dir() {
  cp -r $CONFIG_DIR/ppp /etc
}

load_exec_local_daemons() {
  while read i
  do
    #Unuseful? No, it is needed by repeated save_settings.sh.
    chmod 755 /usr/local/etc/rc.d/$i >/dev/null 2>&1
    # Do all scripts support the restart param? I think no.
    sh /usr/local/etc/rc.d/$i stop >/dev/null 2>&1
    sh /usr/local/etc/rc.d/$i start >/dev/null 2>&1
  done < $CONFIG_DIR/local_daemons.txt
}

load_kbdmap_file() {
  if [ -f $CONFIG_DIR/kbdmap.conf ]; then
    cp $CONFIG_DIR/kbdmap.conf /tmp/kbdmap.conf
    /usr/sbin/kbdcontrol -l /tmp/kbdmap.conf
  fi
}

set_iface() {
  #Stopping the dhclient if present
  DHCLIENT_PID=`ps -x | grep "dhclient -nw $IFACE_NAME" | \
                awk '{print $1}'`
  [ "$DHCLIENT_PID" ] && kill $DHCLIENT_PID >/dev/null 2>&1
  ifconfig $IFACE_NAME $IFACE_ARGS
}

# Find the already mounted partitions in /mnt
echo "Starting"
for i in `mount | grep mnt | cut -d\  -f 3`
do
  IS_PRESENT=`find $i -maxdepth 0 -name 'FreeSBIE_*' -type d | tail -1`
  if [ "${IS_PRESENT}" ]; then
    CONFIG_DIR=$IS_PRESENT
    echo "Reading saved settings from $CONFIG_DIR/${CONFIGFILE_NAME}"
    . $CONFIG_DIR/${CONFIGFILE_NAME}

    #Set the hostname
    [ "$HOSTNAME" ] && set_hostname

    #Network interfaces setup
    for j in 0 1 2 3 4 5 6 7 
    do
      eval IFACE=\$NETWORK_IFACE$j
      if [ "$IFACE" ]; then
        IFACE_NAME=`echo $IFACE | cut -d, -f 1`
        IFACE_ARGS=`echo $IFACE | cut -d, -f 2`
        set_iface
      fi
    done

    #Default router setup
    [ "$DEFAULTROUTER" ] && set_defaultrouter

    #Other settings
    load_rc_conf
    load_kbdmap_file
    load_ppp_dir
    load_exec_local_daemons
  fi
done
