#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
#$Id: save_settings.sh,v 1.6 2005/04/26 17:22:46 rionda Exp $

IFACE_COUNTER="0"
CONFIGFILE_NAME="freesbie_settings.conf"
PARTITION=""

set_defaultrouter() {
  #Set the default router if different from rc.conf setting
  echo "Saving the defaultrouter setting"
  DEFAULTROUTER=`netstat -rn | grep default | awk '{print $2;}'`
  DEF_ROUTER_RC=`grep defaultrouter /etc/rc.conf | cut -d\" -f 2`
  [ "${DEFAULTROUTER}" ] &&                           \
  [ "${DEFAULTROUTER}" != "${DEF_ROUTER_RC}" ] &&     \
  echo "DEFAULTROUTER=\"$DEFAULTROUTER\"" >>          \
                        $PARTITION/$CONFIGFILE_NAME
}

save_x11_config_file() {
  [ -f /etc/X11/xorg.conf ] && cp /etc/X11/xorg.conf $PARTITION/xorg.conf
  [ -f /etc/X11/XFree86.conf ] && cp /etc/X11/XF86Config $PARTITION/XF86Config
}

save_kbdmap_file() {
  [ -f /tmp/kbdmap.conf ] && cp /tmp/kbdmap.conf $PARTITION/kbdmap.conf
}

set_hostname() {
  #Set the hostname if different from rc.conf setting
  echo "Saving the hostname setting"
  HOSTNAME=`hostname`
  [ "${HOSTNAME}" ] && echo "HOSTNAME=\"$HOSTNAME\"" >> \
                                    $PARTITION/$CONFIGFILE_NAME
}
write_iface_config() {
  #$i is the iface id
  IFACE_ARGS=`ifconfig $i | grep inet\ | awk '  \
    {                                           \
      for (i=1; i<= NF; i++)                    \
        print $i;                               \
    }                                           \
  ' | xargs`
  echo "NETWORK_IFACE${IFACE_COUNTER}=\"${i},${IFACE_ARGS}\"" >>  \
  $PARTITION/$CONFIGFILE_NAME
  IFACE_COUNTER=`expr $IFACE_COUNTER + 1`
}

save_iface_config() {
  #Set the nic informations if not dhcp'ed
  echo "Saving network interfaces settings"
  IFACES_NAME=`ifconfig -a | grep flags | cut -d: -f 1`
  for i in $IFACES_NAME
  do
    IFACE_DHCP_TMP=`ps -x | grep "dhclient -nw $i"`
    if [ "${IFACE_DHCP_TMP}" ]; then
      IFACE_DHCP=`echo ${IFACE_DHCP_TMP} | awk '{if ($5 == "dhclient") print $0;}'`
      if [ ! "${IFACE_DHCP}" ]; then
        write_iface_config
      fi
    fi
  done
}

save_rc_conf() {
  echo "Saving rc.conf"
  cp /etc/rc.conf $PARTITION/rc.conf
}

save_exec_local_daemons() {
  echo "Saving local daemons"
  basename `find /usr/local/etc/rc.d/ -name '*sh' -type f -perm -555` \
		> $PARTITION/local_daemons.txt
}

save_ppp_dir() {
  echo "Saving ppp settings"
  mkdir -p $PARTITION/ppp
  cp -r /etc/ppp $PARTITION
}

check_new_or_append() {
  #if the file exists, ask for action
  if [ -f "$PARTITION/$CONFIGFILE_NAME" ]; then
    echo "I found an existing FreeSBIE config file,"
    echo -n "what do you want to do: [O]verwrite, [A]ppend, [Q]uit? "
    read ANS
    if [ -z "${ANS}" ]; then
      echo "------------------"
      check_new_or_append
    fi
    if [ "${ANS}" = "O" -o "${ANS}" = "O" ]; then
      rm ${PARTITION}/${CONFIGFILE_NAME}
      touch ${PARTITION}/${CONFIGFILE_NAME}
    elif [ "${ANS}" = "Q" -o "${ANS}" = "q" ]; then
      exit 0
    fi
  fi
  echo -n "#FreeSBIE save_settings.sh script launched at: " \
      >> ${PARTITION}/${CONFIGFILE_NAME}
  date  >> ${PARTITION}/${CONFIGFILE_NAME}
}

check_location() {
  echo "FreeSBIE save_settings.sh"
  echo "-------------------------"
  echo "Available partitions:"
  GOOD_PARTS=`mount | grep mnt | egrep "ext2fs|msdosfs|ntfs|ufs" | \
              cut -d\  -f 1`
  MOUNTED_PARTS=`mount | grep mnt | egrep "ext2fs|msdosfs|ntfs|ufs" \
                 | cut -d\  -f 3`
  df -h $GOOD_PARTS | sed "s/Filesystem/Partition /" | \
                      sed "s/Mounted\ on/FreeSBIE/"
  echo $DETAILS
  echo "-------------------------"
  echo "To save your data, specify a partition identifier"
  echo -n "using the string in the FreeSBIE column ([Q]uit): "
  read ANS_LOCATION
  [ ! "${ANS_LOCATION}" ] && check_location
  [ "${ANS_LOCATION}" = "Q" -o "${ANS_LOCATION}" = "q" ] && exit 0
  #Check for syntax errors
  IS_OK=1
  for k in $MOUNTED_PARTS
  do
    [ "${k}" = "${ANS_LOCATION}" ] && IS_OK=0
  done
  if [ ! -d ${ANS_LOCATION} -o "${IS_OK}" = "1" ]; then 
    echo "I think you made a syntax error (e.g. mnt -> nmt),"
    echo -n "do you want to retry or exit ([R]etry or a key to exit)? "
    read ANS_RETRY
    if [ "${ANS_RETRY}" = "R" -o "${ANS_RETRY}" = "r" ]; then
      check_location
    else
      exit 0
    fi
  fi
  echo "You selected this partition:"
  df -h $ANS_LOCATION
  echo -n "[C]onfirm, [R]etry or a key to quit: "
  read LAST_ANS
  if [ "$LAST_ANS" = "R" -o "${LAST_ANS}" = "r" ]; then
    check_location
  elif [ "$LAST_ANS" = "C" -o "${LAST_ANS}" = "c" ]; then
    PARTITION=`echo $ANS_LOCATION`
  else
    exit 0
  fi
}

make_subdir() {
  SUBDIR=`date "+%Y%m%d`
  mkdir -p $PARTITION/FreeSBIE_$SUBDIR
  PARTITION=`echo $PARTITION/FreeSBIE_$SUBDIR`
}

check_location
make_subdir
check_new_or_append
set_defaultrouter
set_hostname
save_rc_conf
save_iface_config
save_kbdmap_file
save_x11_config_file
save_exec_local_daemons
save_ppp_dir
