#!/bin/sh
#
# Copyright 2002-2005 G.U.F.I. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#  1.Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
#  2.Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation 
#    and/or other materials provided with the distribution. 
#
# THIS SOFTWARE IS PROVIDED BY G.U.F.I. ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
# EVENT SHALL THE FREEBSD PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
# OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the G.U.F.I.
#
# $Id: freesbieiso2ufs.sh,v 1.6 2005/04/26 17:14:52 rionda Exp $
# vim: set ts=4:

. ./config.sh
. ./.common.sh

set -e -u

DEF_DEV=/dev/da0

if [ ! -f "${PATHISO}" ]; then
	echo "Could not find image ${PATHISO}, aborting"
	exit 1;
fi

echo -n "Device where you want to install FreeSBIE [${DEF_DEV}]: "
read DEVICE

if [ -z "${DEVICE}" ]; then
	DEVICE=$DEF_DEV
fi

if [ ! -c "${DEVICE}" ]; then
	echo "Unable to find ${DEVICE}, please check the pathname"
	exit 1;
fi


DISKMD=/dev/`mdconfig -a -t vnode -f ${PATHISO}`
TEMP2DIR=$PWD/tmp2dir.$$

mkdir $TEMP2DIR
mount_cd9660 ${DISKMD} $TEMP2DIR


dd if=/dev/zero of=${DEVICE} bs=512 count=32
fdisk -BI ${DEVICE}
dd if=/dev/zero of=${DEVICE}s1 bs=512 count=32
bsdlabel -w -B ${DEVICE}s1
bsdlabel ${DEVICE}s1 >> tmp.$$
ed tmp.$$ <<EOF
/a:
s/unused/4.2BSD
wq
EOF
bsdlabel -R ${DEVICE}s1 tmp.$$
rm tmp.$$
newfs ${DEVICE}s1a

TEMPDIR=$PWD/tmpdir.$$
mkdir $TEMPDIR
mount ${DEVICE}s1a $TEMPDIR

cd $TEMP2DIR
find . -depth -print | cpio -pvudm $TEMPDIR


if [ -f "dist/FreeSBIE.root.dist.bz2" ]; then
	bzcat dist/FreeSBIE.root.dist.bz2 | mtree -PUr -p $TEMPDIR
fi

cd ..

umount $TEMPDIR
rmdir $TEMPDIR

umount $TEMP2DIR
rmdir $TEMP2DIR
mdconfig -d -u `echo ${DISKMD} | sed s/\\\/dev\\\/md//g`
